@@ -1,13 +1,13 @@
 package OpenInteract;
 
-# $Id: OpenInteract.pm,v 1.45 2003/04/08 01:16:11 lachoy Exp $
+# $Id: OpenInteract.pm,v 1.50 2003/08/13 03:22:30 lachoy Exp $
 
 use strict;
-use Apache::Constants qw( :common :remotehost );
+use Apache::Constants qw( :common :remotehost :response );
 use Apache::Request;
 use Data::Dumper      qw( Dumper );
 
-$OpenInteract::VERSION  = '1.58';
+$OpenInteract::VERSION  = '1.59';
 
 # Generic separator used in display
 
@@ -58,8 +58,19 @@ sub handler ($$) {
         return $class->bail( $@ );
     }
 
-    if ( $R->{page}{send_file} ) { $class->send_static_file( $R ) }
-    else                         { $class->send_html( $apache, $page, $R ) }
+    if ( $R->{page}{send_file} ) {
+        $class->send_static_file( $R );
+    }
+    elsif ( my $redirect_url = $R->{page}{http_redirect} ) {
+        my $apr = $R->apache;
+        $apr->no_cache(1);
+        $apr->headers_out->set( Location => $redirect_url );
+        $apr->status( REDIRECT );
+        $apr->send_http_header;
+    }
+    else {
+        $class->send_html( $apache, $page, $R );
+    }
 
     $class->cleanup( $R );
     return OK;
@@ -83,12 +94,18 @@ sub setup_request {
     my $STASH_CLASS = $apache->dir_config( 'OIStashClass' );
     unless ( $REQ{ $STASH_CLASS } ) {
         eval "require $STASH_CLASS";
-        die "Cannot require stash class ($STASH_CLASS)!\n" if ( $@ );
+        if ( $@ ) {
+            $apache->child_terminate;
+            die "Cannot require stash class ($STASH_CLASS) -- ",
+                "fatal event, will terminate Apache child\n";
+        }
         $REQ{ $STASH_CLASS }++;
     }
     my $C = $STASH_CLASS->get_stash( 'config' );
     unless ( ref $C and scalar keys %{ $C } ) {
-        die "Cannot find configuration object from stash class ($STASH_CLASS). Cannot continue!";
+        $apache->child_terminate;
+        die "Cannot find configuration object from stash class ($STASH_CLASS) -- ",
+            "fatal event, will terminate Apache child\n";
     }
 
     # Create the base request object that contains other objects and
@@ -97,7 +114,11 @@ sub setup_request {
     my $REQUEST_CLASS = $C->{server_info}{request_class};
     unless ( $REQ{ $REQUEST_CLASS } ) {
         eval "require $REQUEST_CLASS";
-        die "Cannot require request class ($REQUEST_CLASS)!\n" if ( $@ );
+        if ( $@ ) {
+            $apache->child_terminate;
+            die "Cannot require request class ($REQUEST_CLASS) -- ",
+                "fatal event, will terminate Apache child\n";
+        }
         $REQ{ $REQUEST_CLASS }++;
     }
     my $R = $REQUEST_CLASS->instance;
@@ -162,22 +183,26 @@ sub setup_cache {
 # consistent, with the action as the first member.
 
 sub parse_uri {
-    my ( $class, $R ) = @_;
-
-    # Get the Apache::URI object and put it in $R
-
-    # TODO: Do we EVER retrieve the URI object from the stash? Why put it there?
+    my ( $class, $R, $uri ) = @_;
 
     my $apache = $R->apache;
-    my $u = $apache->parsed_uri;
-    $R->stash( 'uri', $u );
+
+    # Get the Apache::URI object and put it in $R
+    unless ( $uri ) {
+        $uri = $apache->parsed_uri;
+    }
+
+    # TODO: Do we EVER retrieve the URI object from the stash? Why put
+    # it there?
+
+    $R->stash( 'uri', $uri );
 
     # Get the path info from the URL and put it in $R; we save it twice
     # so we can shift items from one and still keep the original; we
     # also get the action name from the first item in the path
 
     my $location = $apache->location;
-    my $path = $u->path;
+    my $path = $uri->path;
     $R->DEBUG && $R->scrib( 1, "Original path: ($path)" );
     if ( $location ne '/' ) {
         $path =~ s/^$location//;
@@ -205,7 +230,7 @@ sub parse_uri {
     # directive
 
     $R->{path}{original} = $path;
-    $R->{path}{original} .=  '?' . $u->query  if ( $u->query );
+    $R->{path}{original} .=  '?' . $uri->query  if ( $uri->query );
     $R->DEBUG && $R->scrib( 1, "Original path/query string set to: $R->{path}{original}" );
     return;
 }
@@ -239,11 +264,16 @@ sub check_database {
     my ( $class, $R ) = @_;
     my $db = $R->db( 'main' );
     eval {
-        die unless ( $db );
-        warn "Found item: ", ref( $db ), "\n";
+        unless ( $db ) {
+            $R->apache->child_terminate;
+            die "Database not found -- fatal event, will terminate Apache child\n";
+        }
+        $R->DEBUG && warn "Found item: ", ref( $db ), "\n";
         $db->ping;
     };
     if ( $@ ) {
+        $R->apache->child_terminate;
+        $R->scrib( 0, "Cannot ping database -- fatal event, will terminate Apache child" );
         my $error_msg = $R->throw({ code => 11 });
         $class->send_html( $R->apache, $error_msg, $R );
         die OK . "\n";
@@ -303,10 +333,66 @@ sub setup_authentication {
         $error_msg = "Authentication cannot be setup! Please ensure 'auth' " .
                      "is setup in your server configuration under 'system_alias'";
     }
-    return undef unless ( $error_msg );
+    if ( $error_msg ) {
+        $class->send_html( $R->apache, $error_msg, $R );
+        die OK . "\n";
+    }
 
-    $class->send_html( $R->apache, $error_msg, $R );
-    die OK . "\n";
+    my $login_info = $R->CONFIG->{login};
+    return undef unless ( $login_info->{required} );
+    return undef if ( $R->{auth}{logged_in} );
+    my $url_requires_login =
+        $class->url_requires_login( $R->{path}{original},
+                                    $login_info->{required_skip} );
+    return undef unless ( $url_requires_login );
+    return $class->required_login_not_found( $R );
+}
+
+
+# True if the URL requires login (that is, it DOESN'T match a member
+# of \@url_to_skip)
+
+sub url_requires_login {
+    my ( $class, $url, $url_to_skip ) = @_;
+    return 1 unless ( $url_to_skip );
+    my @urls_to_check = ( ref $url_to_skip eq 'ARRAY' )
+                          ? @{ $url_to_skip } : ( $url_to_skip );
+    my $url_skip = 0;
+    foreach my $url_check ( @urls_to_check ) {
+        next unless ( $url_check );
+        $url_skip++ if ( $url =~ /$url_check/ );
+    }
+    return ( ! $url_skip );
+}
+
+# Reset the URL to 'login.required_url' (could this have side
+# effects?)
+
+sub required_login_not_found {
+    my ( $class, $R ) = @_;
+    my $required_url = $R->CONFIG->{login}{required_url};
+    unless ( $required_url ) {
+        $R->scrib( 0, "You have 'login.required' enabled so I'm ensuring ",
+                      "that all users have a login, but you don't have ",
+                      "'login_required_url' set to a URL where I should ",
+                     "send them. Ignoring login requirement setting..." );
+        return undef;
+    }
+    my $apache = $R->apache;
+    my $host = $R->{server_name};
+    my $port = $apache->get_server_port;
+    my $full_port = ( $port == 80 ) ? '' : ":$port";
+    my $full_url = join( '', 'http://', $host, $full_port, $required_url );
+    my $uri = Apache::URI->parse( $apache, $full_url );
+    $R->DEBUG && $R->scrib( 1, "Resetting request URL to '$full_url' since ",
+                               "login required and none found" );
+
+    # This is available for the login page to set in a hidden variable
+    # of its choosing; it's also used in the template plugin method
+    # 'return_url()' called by the login box...
+
+    $R->{path}{login_fail} = $R->{path}{original};
+    return $class->parse_uri( $R, $uri );
 }
 
 
@@ -523,6 +609,17 @@ groups the user belongs to, plus set for this request whether the user
 is an administrator. Also run the custom authentication handler as
 defined in the server configuration.
 
+If the 'login.required' server configuration key is a true value we
+ensure that there's a legitimate user logged in. If there's no login
+we call the 'url_requires_login()' class method passing the requested
+URL and the arrayref of URLs in the 'login.required_skip' server
+configuration key.
+
+If that method indicates that the URL does require a login we call the
+class method 'required_login_not_found()', passing C<$R> as the sole
+argument. That method is responsible for resetting the URL in the
+request to that specified in 'login.required_url'.
+
 Return: nothing
 
 On error: Send error information to user via C<send_html()> then
